from pathlib import Path
from random import choice
from typing import Optional
from cosa.transform import k_representative_pallette, elastic_transform, jpeg
from PIL import Image
import numpy as np

FUNCTIONS = {
    "k_rep": k_representative_pallette,
    "elastic": elastic_transform,
    "jpeg": jpeg,
}


class Cosa:
    """Main `cosa` class that has the functionality to
    read, transform and save images.
    """

    def __init__(self):
        self.image = None
        self.transformed = None

    def read(self, input_path: str) -> None:
        """Load an image, given a path.
        Args:
            input_path (str): The path to the image that
                will be transformed.

        Returns:
            None
        """
        input_path = Path(input_path)
        if input_path.is_file():
            self.image = np.asarray(Image.open(input_path))
        else:
            raise ValueError(f"File {input_path} does not exist!")

    def transform(self, func: Optional[str] = None, **args) -> None:
        """Transform a loaded image with a given
        functions. If the function is not specified
        one will be chosen randomly.

        Args:
            func (Optional[str]): The name of the function
                that will be applied as a transformation to the
                image.
            **args

        Returns:
            None
        """
        funcs_list = list(FUNCTIONS.keys())
        if func not in funcs_list and func is not None:
            raise ValueError(f"{func} function not supported.")
        if not func:
            func = choice(funcs_list)
        self.transformed = FUNCTIONS[func](image=self.image, **args)

    def write(self, output_path: str) -> None:
        """Write an image in a given path.

        Args:
            output_path (str): The path to the image that
                will be saved after the transformation.

        Returns:
            None
        """
        output_path = Path(output_path)
        output_path.parent.mkdir(parents=True, exist_ok=True)
        if self.transformed is None:
            raise ValueError(
                "There is no image to write! Make sure to load & transform one first."
            )
        self.transformed.save(output_path)



from setuptools import setup

setup(
    name="cosabot",
    version="0.0.6",
    description="Computer Optic Semantics",
    packages=["cosa"],
    author="Alfredo Lozano",
    author_email="lozanoa94@gmail.com",
    install_requires=["scipy", "scikit-image", "scikit-learn", "numpy"],
    zip_safe=False,
)



import random
from pathlib import Path
import numpy as np
from numpy.random import RandomState
from sklearn.cluster import KMeans
from PIL import Image
from scipy.ndimage.filters import gaussian_filter
from scipy.ndimage.interpolation import map_coordinates


def k_representative_pallette(
    image: np.ndarray, num_clusters: int = random.randint(4, 5)
) -> Image:
    """Cluster pixels into num_clusters representative colors
    and transform image into to this discrete palette
    """
    ysize = image.shape[0]
    xsize = image.shape[1]
    new_image = np.reshape(image, (xsize * ysize, 3))
    model = KMeans(n_clusters=num_clusters, random_state=42).fit(new_image)
    colors = []
    for i in range(num_clusters):
        colors.append(np.median(new_image[model.labels_ == i], axis=0))
        colors[i] = colors[i] / 255
    new_image = np.zeros((xsize * ysize, 3))
    for i in range(num_clusters):
        slicer = model.labels_ == i
        new_image[slicer] = colors[i]
    new_image = np.reshape(new_image, (ysize, xsize, 3))
    new_image = Image.fromarray((new_image * 255).astype(np.uint8))
    return new_image


def elastic_transform(
    image: np.ndarray,
    alpha: int = random.randint(900, 1000),
    sigma: int = random.randint(4, 8),
    random_state: RandomState = None,
) -> Image:
    """Elastic deformation of images as described in [Simard2003]
    Simard, Steinkraus and Platt, "Best Practices for
    Convolutional Neural Networks applied to Visual Document
    Analysis", in Proc. of the International Conference on
    Document Analysis and Recognition, 2003.
    """
    if random_state is None:
        random_state = RandomState(None)
    shape = image.shape
    dx = (
        gaussian_filter(
            (random_state.rand(*shape) * 2 - 1), sigma, mode="constant", cval=0
        )
        * alpha
    )
    dy = (
        gaussian_filter(
            (random_state.rand(*shape) * 2 - 1), sigma, mode="constant", cval=0
        )
        * alpha
    )
    x, y, z = np.meshgrid(
        np.arange(shape[1]),
        np.arange(shape[0]),
        np.arange(shape[2]),
    )
    indices = (
        np.reshape(y + dy, (-1, 1)),
        np.reshape(x + dx, (-1, 1)),
        np.reshape(z, (-1, 1)),
    )
    distored_image = map_coordinates(image, indices, order=1, mode="reflect")
    dist_reshaped = distored_image.reshape(image.shape)
    dist_reshaped = Image.fromarray(dist_reshaped)
    return dist_reshaped


def jpeg(image: np.ndarray, iterations: int = 100) -> Image:
    """Applies a JPEG compression `iterations` times.
    This is inspired by the `JPEG Bot`. For more info,
    see:
    `https://mikewatson.me/bots/JPEGBot`
    """
    im = Image.fromarray(image)
    temp_file_path = "/tmp/jpeg_transoform_cosa.jpeg"
    temp_file_path = Path(temp_file_path)
    temp_file_path.parent.mkdir(parents=True, exist_ok=True)
    im.save(temp_file_path, format="JPEG", quality=100)
    for i in range(100):
        im = Image.open(temp_file_path)
        im.save(temp_file_path, format="JPEG", quality=115 - i)
    return Image.open(temp_file_path)
